/*
    ChibiOS/RT - Copyright (C) 2006-2007 Giovanni Di Sirio.

    This file is part of ChibiOS/RT.

    ChibiOS/RT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    ChibiOS/RT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ch.h>
#include <signal.h>

#include "board.h"

/*
 * Hardware initialization goes here.
 * NOTE: Interrupts are still disabled.
 */
void hwinit(void) {

  /*
   * Clock sources setup.
   */
  DCOCTL  = VAL_DCOCTL;
  BCSCTL1 = VAL_BCSCTL1;
  BCSCTL2 = VAL_BCSCTL2;

  /*
   * I/O ports initialization.
   */
  P1OUT = VAL_P1OUT;
  P1DIR = VAL_P1DIR;
  P1SEL = VAL_P1SEL;

  P2OUT = VAL_P2OUT;
  P2DIR = VAL_P2DIR;
  P2SEL = VAL_P2SEL;

  P3OUT = VAL_P3OUT;
  P3DIR = VAL_P3DIR;
  P3SEL = VAL_P3SEL;

  P4OUT = VAL_P4OUT;
  P4DIR = VAL_P4DIR;
  P4SEL = VAL_P4SEL;

  P5OUT = VAL_P5OUT;
  P5DIR = VAL_P5DIR;
  P5SEL = VAL_P5SEL;

  P6OUT = VAL_P6OUT;
  P6DIR = VAL_P6DIR;
  P6SEL = VAL_P6SEL;

  /*
   * Timer 0 setup, uses SMCLK as source.
   */
  TACCR0 = SMCLK / CH_FREQUENCY - 1;    /* Counter limit.               */
  TACTL = TACLR;                        /* Clean start.                 */
  TACTL = TASSEL_2 | MC_1;              /* Src=SMCLK, cmp=TACCR0.       */
  TACCTL0 = CCIE;                       /* Interrupt on compare.        */
}

interrupt(TIMERA0_VECTOR) tmr0irq(void) {

  chSysIRQEnterI();
  chSysTimerHandlerI();
  chSysIRQExitI();
}
